package com.shellinabox.console;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Hashtable;
import java.util.zip.GZIPInputStream;


class VectorFont {

    private static final char    UNKNOWN_GLYPH  = '\uFFFD';


    private Hashtable fontData;
    private int[][]   indexedFontData;
    private int       cellWidth;
    private int       cellHeight;
    private int       baseLine;
    private int       numGlyphs;
    private int[]     unknownGlyph;


    private static Hashtable sharedFonts        = new Hashtable();


    private VectorFont(URL fontURL) throws IOException {
        InputStream fontInputStream = null;
        try {
            try {
                // Load compressed font if available and supported by the
                // Java VM. Otherwise fall back to the uncompressed font.
                URL fontURLGZip = new URL(fontURL.toString() + ".gz");
                fontInputStream = fontURLGZip.openStream();
                fontInputStream = getInputStream(fontInputStream);
            } catch (Exception e) {
                // JDK 1.0.2
                try {
                    fontInputStream.close();
                } catch (Exception ex) {
                }
                fontInputStream = fontURL.openStream();
            }
            cellWidth            = fontInputStream.read();
            cellHeight           = fontInputStream.read();
            baseLine             = fontInputStream.read();
            numGlyphs            = fontInputStream.read();
            numGlyphs            = 256*numGlyphs + fontInputStream.read();
            fontData             = new Hashtable(3*numGlyphs);
            indexedFontData      = new int[256][];
            int[] scratchPad     = new int[256];
            int glyphsRead;
        eof:for (glyphsRead = 0; glyphsRead < numGlyphs; glyphsRead++) {
                int i = fontInputStream.read(); if (i < 0) break eof;
                int j = fontInputStream.read(); if (j < 0) break eof;
                Character uniCode = new Character((char)(i*256+j));
                int idx = 0;
                for (;;) {
                    i = fontInputStream.read(); if (i < 0) break eof;
                    j = fontInputStream.read(); if (j < 0) break eof;
                    if (i == 0x80 && j == 0x80)
                        break;
                    scratchPad[idx++] = (int)((i/16) & 0x7);
                    scratchPad[idx++] = (int)( i     & 0xF);
                    if (j >= 0x80) {
                        scratchPad[idx++]=(int)(((j/16)&0x7)-((i/16)&0x7)+101);
                        scratchPad[idx++]=(int)(( j    &0xF)-( i    &0xF)+1);
                    } else {
                        scratchPad[idx++]=(int)((j/16) & 0x7);
                        scratchPad[idx++]=(int)( j     & 0xF);
                    }
                    if ((i & 0x80) != 0)
                        break;
                }
                int[] coords = new int[idx];
                System.arraycopy(scratchPad,0,coords,0,idx);
                if (uniCode.charValue() < 256)
                    indexedFontData[uniCode.charValue()] = coords;
                fontData.put(uniCode,coords);
            }
            if (glyphsRead != numGlyphs)
                throw new IOException();
        } catch (IOException e) {
            throw new IOException("Failed to read font: " + fontURL);
        } finally {
            try {
                fontInputStream.close();
            } catch (Exception e) {
            }
        }
        unknownGlyph = (int [])fontData.get(new Character(UNKNOWN_GLYPH));
        return;
    }


    /**
     * Clutch to dynamically load and access GZIPInputStream. This is
     * surprisingly difficult to get right, because the original Java
     * bytecode verifier was very picky about not loading classes if
     * they depended on other non-existant class files.
     */
    private InputStream getInputStream(InputStream inputStream)
                                                           throws IOException {
        try {
            Class wrapperHelperClass =Class.forName(VectorFont.class.getName()+
                                                    "$Helper");
            VectorFont wrapperHelper =
                                  (VectorFont)wrapperHelperClass.newInstance();
            return(wrapperHelper.helper(inputStream));
        } catch (Exception e) {
            throw new IOException();
        }
    }


    protected VectorFont() {
        return;
    }


    InputStream helper(InputStream inputStream) throws IOException {
        return(null);
    }


    static class Helper extends VectorFont {

        InputStream helper(InputStream inputStream) throws IOException {
            return(new GZIPInputStream(inputStream));
        }
    }


    synchronized static public VectorFont getSharedFont(URL url) {
        VectorFont font = (VectorFont)sharedFonts.get(url);
        if (font != null) {
            return(font); }
        try {
            font = new VectorFont(url);
        } catch (IOException e) {
            // don't care
        }
        if (font != null)
            sharedFonts.put(url,font);
        return(font);
    }

    
    public Hashtable getFontData() {
        return(fontData);
    }


    public int[][] getIndexedFontData() {
        return(indexedFontData);
    }


    public int getCellWidth() {
        return(cellWidth);
    }


    public int getCellHeight() {
        return(cellHeight);
    }


    public int getBaseLine() {
        return(baseLine);
    }


    public int getNumGlyphs() {
        return(numGlyphs);
    }


    public int[] getUnknownGlyph() {
        return(unknownGlyph);
    }
}

/*
 * Local variables:
 * mode: java
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End: */
