package com.shellinabox.dialogs;

import java.awt.Button;
import java.awt.Component;
import java.awt.Event;
import java.awt.Font;
import java.awt.Label;
import java.awt.MenuItem;
import java.awt.Panel;
import java.awt.PopupMenu;
import java.awt.Rectangle;
import java.util.Vector;


public class ComboBox extends Panel {

    private Label     label;
    private Button    button;
    private ComboBox  menuOwner;
    private boolean   emptyMenu = true;
    private Vector    entries;
    private int       currentEntry;


    protected ComboBox() {
    }


    public ComboBox(String label) {
        super();
        this.label = new Label(label);
        button     = new Button();
        try {
            Class helperClass = Class.forName(ComboBox.class.getName() +
                                              "$Helper");
            menuOwner = (ComboBox)helperClass.newInstance();
            menuOwner.initMenu(this);
        } catch (Exception e) {
            menuOwner = this;
            menuOwner.initMenu(this);
        }
        add(this.label);
        add(button);
        setLayout(null);
        return;
    }


    public void reshape(int x,int y,int w,int h) {
        super.reshape(x,y,w,h);
        int labelWidth  = w/2;
        int buttonWidth = w/2;
        label.reshape(0,0,labelWidth,h);
        button.reshape(labelWidth,0,buttonWidth,h);
        return;
    }


    public void setFonts(Font labelFont,Font buttonFont) {
        label.setFont(labelFont);
        button.setFont(buttonFont);
        menuOwner.setMenuFont(buttonFont);
        return;
    }


    public String getLabel() {
        return(button.getLabel());
    }


    public void setLabel(String label) {
        button.setLabel(label);
        if (entries != null) {
            for (int i = 0; i < entries.size(); i++) {
                if (label.equals((String)entries.elementAt(i))) {
                    currentEntry = i;
                    break;
                }
            }
        }
        return;
    }


    public void add(String menuString) {
        if (emptyMenu) {
            button.setLabel(menuString);
            emptyMenu = false;
        }
        menuOwner.addMenuEntry(menuString);
        return;
    }


    public boolean handleEvent(Event ev) {
        if (ev.id == Event.ACTION_EVENT) {
            if (ev.target instanceof MenuItem) {
                button.setLabel((String)ev.arg);
            } else if (ev.target instanceof Button) {
                Rectangle bounds = bounds();
                menuOwner.showMenu(this,bounds.width/2+2,bounds.height-2);
            }
        }
        return(super.handleEvent(ev));
    }


    protected void showMenu(Component origin,int x,int y) {
        // JDK 1.0.2 doesn't support popup menus, cycle through entries instead
        currentEntry = (currentEntry + 1) % entries.size();
        button.setLabel((String)entries.elementAt(currentEntry));
        return;
    }


    protected void initMenu(Component parent) {
        entries      = new Vector();
        currentEntry = 0;
        return;
    }


    protected void setMenuFont(Font font) {
        return;
    }


    protected void addMenuEntry(String entry) {
        entries.addElement(entry);
        if (entry.equals(button.getLabel()))
            currentEntry = entries.size() - 1;
        return;
    }


    static class Helper extends ComboBox {

        private PopupMenu popupMenu;


        public Helper() {
            return;
        }


        protected void showMenu(Component origin,int x,int y) {
            popupMenu.show(origin,x,y);
            return;
        }


        protected void initMenu(Component parent) {
            popupMenu = new PopupMenu();
            parent.add(popupMenu);
            return;
        }


        protected void setMenuFont(Font font) {
            popupMenu.setFont(font);
            return;
        }


        protected void addMenuEntry(String entry) {
            popupMenu.add(entry);
            return;
        }
    }
}

/*
 * Local variables:
 *  mode: java
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
