package com.shellinabox.dialogs;

import com.shellinabox.ShellInABox;
import com.shellinabox.persistent.Configuration;
import java.awt.Button;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.Panel;
import java.util.Enumeration;
import java.util.Vector;


public class ConfigurationDialog extends Frame {

    public static final int     LINUX                  = 0;
    public static final int     ANSI                   = 1;
    public static final int     VT102                  = 2;
    public static final int     XTERM                  = 3;
    public static final int     XTERM_COLOR            = 4;
    public static final int     DELETE                 = 0;
    public static final int     BACKSPACE              = 1;
    public static final int     LARGE                  = 0;
    public static final int     MEDIUM                 = 1;
    public static final int     SMALL                  = 2;
    public static final int     SYSTEM                 = 0;
    public static final int     VISUAL                 = 1;
    public static final int     FANCY                  = 2;
    public static final int     INTERNAL               = 0;
    public static final int     NATIVE                 = 1;

    private static final String TITLE                  = "Preferences...";
    private static final String PREFIX                 = "P";

    private static final int    TERMINALTYPE           = 0;
    private static final int    CURSORKEYTYPE          = 1;
    private static final int    BACKSPACETYPE          = 2;
    private static final int    FONTSIZE               = 3;
    private static final int    BELLTYPE               = 4;
    private static final int    CLIPBOARDTYPE          = 5;
    
    private static final int    OK                     = 0;
    private static final int    CANCEL                 = 1;
    private static final int    SAVE                   = 2;

    private static final String[]   comboLabels        = {
         "Terminal type:", "Cursor keys:", "Backspace sends:",
         "Font size:", "Bell type:", "Clipboard type:" };
    private static final String[]   comboNames         = {
        "TT", "CK", "BS", "FS", "BT", "CT" };
    private static final String[][] pullDownMenuEntries= {
        {"Linux", "ANSI", "VT 102", "XTerm", "XTerm (color)" },
        {"Linux", "ANSI", "VT 102", "XTerm", "XTerm (color)" },
        {"Delete", "Backspace" },
        {"Large", "Medium", "Small" },
        {"System", "Visual", "Audio" },
        {"Internal", "Native" } };
    private static final String[]   buttonLabels       = {
        "OK", "Cancel", "Save" };

    private static final int    SEP                    = 5;
    private static final int    WIDGETHEIGHT           = 30;
    private static final int    ENTRIES                = comboLabels.length;
    private static final int    BUTTONS                = buttonLabels.length;
    private static final int    WIDTH                  = 450;
    private static final int    HEIGHT                 = (1+ENTRIES/2)*
                                                      (WIDGETHEIGHT+SEP)+4*SEP;

    private Configuration configuration;
    private Panel         panel;
    private ComboBox[]    comboBoxes;
    private Button[]      buttons;
    private Dimension     frameSize;
    private int[]         comboSettings;
    private ShellInABox   applet;

    
    public ConfigurationDialog(ShellInABox applet,Configuration configuration){
        super(TITLE);
        this.applet        = applet;
        this.configuration = configuration;
        frameSize          = new Dimension(WIDTH,HEIGHT);
        setResizable(false);
        setLayout(null);
        resize(WIDTH,HEIGHT);
        pack();
        Insets insets      = insets();
        panel              = new Panel();
        int width          = WIDTH+insets.left+insets.right;     
        int height         = HEIGHT+insets.top+insets.bottom;
        frameSize          = new Dimension(width,height);
        panel.reshape(0,0,width,height);
        resize(width,height);
        panel.setLayout(null);
        add(panel);
        Font labelFont     = new Font("Serif",Font.BOLD,12);
        Font menuFont      = new Font("SansSerif",Font.PLAIN,12);
        comboBoxes         = new ComboBox[ENTRIES];
        comboSettings      = new int[ENTRIES];
        for (int i = 0; i < ENTRIES; i++) {
            comboBoxes[i]  = new ComboBox(comboLabels[i]);
            comboBoxes[i].reshape(insets.left +
                                  ((i & 1) == 1 ? (WIDTH+SEP)/2 : 2*SEP),
                                  insets.top + (2+i/2)*SEP+(i/2)*WIDGETHEIGHT,
                                  (WIDTH-5*SEP)/2,WIDGETHEIGHT);
            comboBoxes[i].setFonts(labelFont,menuFont);
            for (int j = 0; j < pullDownMenuEntries[i].length; j++) {
                comboBoxes[i].add(pullDownMenuEntries[i][j]);
            }
            panel.add(comboBoxes[i]);
        }
        buttons           = new Button[BUTTONS];
        for (int i = 0; i < BUTTONS; i++) {
            buttons[i]    = new Button(buttonLabels[i]);
            buttons[i].reshape(insets.left +
                               2*SEP+i*(SEP+(WIDTH-(3+BUTTONS)*SEP)/BUTTONS),
                               insets.top +
                               (3+ENTRIES/2)*SEP+(ENTRIES/2)*WIDGETHEIGHT,
                               (WIDTH-(3+BUTTONS)*SEP)/BUTTONS,WIDGETHEIGHT);
            buttons[i].setFont(labelFont);
            panel.add(buttons[i]);
        }
        if (configuration == null)
            buttons[SAVE].enable(false);
        else {
            for (Enumeration enum = configuration.keys();
                 enum.hasMoreElements(); ) {
                String key = (String)enum.nextElement();
                if (key.startsWith(PREFIX))
                    for (int j = 0; j < comboNames.length; j++)
                        if (key.equals(PREFIX + comboNames[j])) {
                            String value = (String)configuration.get(key);
                            for (int k = 0; k < pullDownMenuEntries[j].length;
                                 k++)
                                if (pullDownMenuEntries[j][k].equals(value)) {
                                    comboSettings[j] = k;
                                    break;
                                }
                            break;
                        }
            }
            restoreSettings();
        }
        saveSettings(false);
        return;
    }


    public boolean handleEvent(Event ev) {
        switch (ev.id) {
        case Event.WINDOW_MOVED:
            // There is an ugly bug in Netscape/X11 that shrinks non-
            // resizable windows when an attempt is made to change their
            // size interactively. Undo the changes when they happen.
            super.handleEvent(ev);
            resize(frameSize);
            return(true);
        case Event.WINDOW_DESTROY:
            hide();
            restoreSettings();
            break;
        case Event.ACTION_EVENT:
            if (ev.target == buttons[OK]) {
                hide();
                saveSettings(false);
            } else if (ev.target == buttons[CANCEL]) {
                hide();
                restoreSettings();
            } else if (ev.target == buttons[SAVE]) {
                hide();
                saveSettings(true);
            }
            break;
        default:
            break;
        }
        return(super.handleEvent(ev));
    }


    private void saveSettings(boolean updatePersistents) {
        for (int i = 0; i < ENTRIES; i++) {
            String label = comboBoxes[i].getLabel();
            for (int j = 0; j < pullDownMenuEntries[i].length; j++)
                if (pullDownMenuEntries[i][j].equals(label)) {
                    comboSettings[i] = j;
                    break;
                }
        }
        if (updatePersistents) {
            // Remove all old settings with our prefix; this allows for
            // changing parameters between versions. Under JDK 1.0.2 this
            // has to be a 2 pass process.
            Vector toBeRemoved = new Vector();
            for (Enumeration enum = configuration.keys();
                 enum.hasMoreElements(); ) {
                String key = (String)enum.nextElement();
                if (key.startsWith(PREFIX))
                    toBeRemoved.addElement(key);
            }
            for (Enumeration enum = toBeRemoved.elements();
                 enum.hasMoreElements(); )
                configuration.remove(enum.nextElement());
            for (int i = 0; i < ENTRIES; i++)
                configuration.put(PREFIX + comboNames[i],
                                  pullDownMenuEntries[i][comboSettings[i]]);
            configuration.updatePersistents();
        }
        applet.setTerminalType(comboSettings[0]);
        applet.setCursorKeyType(comboSettings[1]);
        applet.setBackspaceType(comboSettings[2]);
        applet.setFontSize(comboSettings[3]);
        applet.setBellType(comboSettings[4]);
        applet.setClipboardType(comboSettings[5]);
        return;
    }


    private void restoreSettings() {
        for (int i = 0; i < ENTRIES; i++)
            comboBoxes[i].setLabel(pullDownMenuEntries[i][comboSettings[i]]);
        return;
    }

}

/*
 * Local variables:
 *  mode: java
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
