package com.shellinabox.tunnel;

import java.awt.Dimension;
import java.net.URL;


public class Tunnel implements GuiObject, TunnelConstants {

    private HttpThread       readThread;
    private SubmitDataThread submitThread;
    private GuiObject        guiObject;
    private URL              cgiURL;


    public Tunnel(URL cgiURL,GuiObject guiObject) {
        this.cgiURL    = cgiURL;
        this.guiObject = guiObject;
        return;
    }


    public void connect(String terminal,boolean backspaceSendsDelete,
                        Dimension charSize,Dimension pixelSize)
                                                       throws TunnelException {
        if (readThread != null)
            throw new TunnelException("Already connected");
        readThread   = new HttpThread(cgiURL,this,terminal,
                                      backspaceSendsDelete,charSize,pixelSize);
        submitThread = new SubmitDataThread(cgiURL,readThread);
        return;
    }


    public void disconnect() throws TunnelException {
        try {
            submitThread.disconnect();
        } catch (NullPointerException e) {
            // Don't care
        }
        submitThread = null;
        try {
            readThread.disconnect();
        } catch (NullPointerException e) {
            // Don't care
        }
        readThread = null;
        return;
    }


    public void submitText(String text) throws TunnelException {
        try {
            submitThread.submit(text);
        } catch (NullPointerException e) {
            throw new TunnelException("Connection closed...");
        }
        return;
    }


    public boolean isConnected() {
        return(readThread != null);
    }


    public void receiveDataEvent(String data) {
        if (isConnected()) {
            guiObject.receiveDataEvent(data);
        }
        return;
    }


    public void disconnectEvent() {
        if (isConnected()) {
            guiObject.disconnectEvent();
            try {
                disconnect();
            } catch (TunnelException e) {
                // Don't care
            }
        }
        return;
    }

}

/*
 * Local variables:
 *  mode: java
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
