#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "shellinaboxd.h"
#include "util.h"


static const char *contentLength;
static const char *contentType;
static const char *pathInfo;
static const char *queryString;
static const char *requestMethod;
static const char *scriptName;
static const char *serverName;
static const char *serverPort;


static void abortCGI(int errCode,char *msg,...)
{
    char    *buf;
    va_list argPtr;
    
    va_start(argPtr,msg);
    buf = vAbortCGI(errCode,msg,argPtr);
    va_end(argPtr);
    fwrite(buf,strlen(buf),1,stdout);
    free(buf);
    exit(0);
}


static void initGlobalVariables(const char *appPath) {
    static const struct Table {
        char const ** const var;
        char * const        name; } *tPtr, table[] = {
            {&contentLength,"CONTENT_LENGTH"},
            {&contentType,  "CONTENT_TYPE"},
            {&pathInfo,     "PATH_INFO"},
            {&queryString,  "QUERY_STRING"},
            {&requestMethod,"REQUEST_METHOD"},
            {&scriptName,   "SCRIPT_NAME"},
            {&serverName,   "SERVER_NAME"},
            {&serverPort,   "SERVER_PORT"} };
    char *pPtr;
    int  i;

    // Initialize parameters that can be read from the environment
    for (i = sizeof(table)/sizeof(struct Table), tPtr = table; i--; tPtr++ ) {
        char *env = getenv(tPtr->name);
        *(tPtr->var) = env == NULL ? "" : env; }
    pPtr = malloc(strlen(serverName) +
                 strlen(serverPort) +
                 strlen(scriptName) +
                  11);
    return;
}


static void processPostData()
{
    int  contentLen;
    char *buf;
    int  sessionId;
    int  sequenceNumber;

    // Verify that we really received a request using our protocol
    if (strcasecmp(contentType,WLCONTENTTYPE))
        abortCGI(400,"Illegal Request\r\n"
                     "Unsupported content type: \"%s\"",contentType);
    contentLen = atoi(contentLength);
    if (contentLen < 8)
      eofErr:
        abortCGI(400,"Illegal Request\r\n"
                     "Data packet too short!");
    // Read the entire request into memory
    buf = calloc(1+(BLOCKSIZE > contentLen ? BLOCKSIZE : contentLen),1);
    if (fread(buf,contentLen,1,stdin) != 1)
        goto eofErr;
    // Parse the request
    sessionId      = ((int)((unsigned char *)buf)[0] << 24) |
                     ((int)((unsigned char *)buf)[1] << 16) |
                     ((int)((unsigned char *)buf)[2] <<  8) |
                      (int)((unsigned char *)buf)[3];
    sequenceNumber = ((int)((unsigned char *)buf)[4] << 24) |
                     ((int)((unsigned char *)buf)[5] << 16) |
                     ((int)((unsigned char *)buf)[6] <<  8) |
                      (int)((unsigned char *)buf)[7];
    if (contentLen >= (8+strlen(CONNECT)) &&
        !memcmp(buf+8,CONNECT,strlen(CONNECT))) {
        char data[] = "Content-type: "WLCONTENTTYPE"\r\n"
                      "\r\n"
                      CONNECT"\000\000\000\000\000\000\000\000";
        fwrite(data,sizeof(data),1,stdout); }
    else if (contentLen >= (8+strlen(POLL)) &&
             !memcmp(buf+8,POLL,strlen(POLL)))
        if (sequenceNumber == 1)
            printf("Content-type: "WLCONTENTTYPE"\r\n"
                   "\r\n"
                   DATA
                   "\r\n"
                   "               \033[34mCongratulations!\033[m\r\n"
                   "\r\n"
                   "If you can read this, then your client software\r\n"
                   "is compatible with ShellInABox.\r\n"
                   "\r\n"
                   "The applet successfully  established a  connec-\r\n"
                   "tion  with the  remote  server and  simulated a\r\n"
                   "user logging in.\r\n");
        else
            abortCGI(400,"Connection closed!\r\n"
                     "Connection closed by foreign host.");
    else
        abortCGI(400,"Unknown request!\r\n"
                 "This request is not supported by the demo CGI.");
    // Done
    exit(0);
    return;
}


int main(int argc,char *argv[])
{
    initGlobalVariables(argv[0]);
    if (!strcmp(requestMethod,"POST")) {
        // All sessions are encapsulated in POST data
        processPostData();
        return(1); }
    abortCGI(400,"Illegal Request\r\n"
                 "<code>REQUEST_METHOD</code> has to be <code>POST</code>");
    return(1);
}

/*
 * Local variables:
 * mode: c
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 */
