#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <grp.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/poll.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/wait.h>
#include <syslog.h>
#include <termios.h>
#include <time.h>
#include <unistd.h>

#include "shellinaboxd.h"
#include "util.h"


char *vAbortCGI(int errCode,char *msg,va_list argPtr)
{
    int     len,sz;
    char    *buf,*rc;
    char    *html;
    
    len = 1024;
    buf = malloc(len);
    // Print message to buffer, dynamically increase size if neccessary
    for (;;) {
        sz = vsnprintf(buf,len-1,msg,argPtr);
        if (sz < len-2)
            break;
        len += 1024;
        buf = realloc(buf,len); }
    // Create HTTP error message
    html = "Content-type: text/html\r\n"
           "Status: %d %s"
           "\r\n"
           "\r\n"
           "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\r\n"
           "<html>\r\n"
           "  <head>\r\n"
           "    <title>%s</title>\r\n"
           "  </head>\r\n"
           "  <body>\r\n"
           "    <h1>%s</h1>\r\n"
           "    %s\r\n"
           "  </body>\r\n"
           "</html>\r\n";
    len = strcspn(buf,"\r\n");
    sz  = len + strspn(buf + len,"\r\n");
    rc  = malloc(strlen(html) + 25 + 3*len + strlen(buf+sz));
    buf[len] = '\000';
    sprintf(rc,html,errCode,buf,buf,buf,buf+sz);
#ifdef LOGPROTOCOLERRORS
    // Write message to system log
    syslog(LOG_DAEMON|LOG_NOTICE,"%s",buf);
    for (;;) {
        int i;
        len = sz + strcspn(buf+sz,"\r\n");
        i   = len + sz + strspn(buf + len + sz,"\r\n");
        buf[len] = '\000';
        if (!buf[sz])
            break;
        syslog(LOG_DAEMON|LOG_NOTICE,"%s",buf+sz);
        sz  = i; }
#endif
    free(buf);
    return(rc);
}


char *resolveSymbolicLinks(const char *path)
{
    char *buf;
    char *ptr;
    int  linkBufLen = 256;
    char *linkBuf   = malloc(linkBufLen);
    int  loopCount  = 10;

    // Make path absolute if given a relative path
    if (*path == '/')
        buf = strdup(path);
    else {
        int   curDirLen = 256;
        char *curDir    = malloc(curDirLen+1);
        for (;;) {
            if (getcwd(curDir,curDirLen) != NULL)
                break;
            if (errno != ERANGE) {
                free(curDir);
                return(NULL); }
            curDirLen += 256;
            curDir     = realloc(curDir,curDirLen+1); }
        curDirLen      = strlen(curDir);
        buf = realloc(curDir,curDirLen + strlen(path) + 2);
        if (!curDirLen || buf[curDirLen-1] != '/')
            strcat(buf,"/");
        strcat(buf,path); }
    // Traverse the path and follow symbolic links
    for (ptr = buf+1; *ptr; ) {
        int len = (ptr - buf) + strcspn(ptr,"/");
        int ch  = buf[len];
        int rc,i;
        buf[len] = '\000';
        // Read link and increase buffer size if neccessary
        for(;;) {
            rc = readlink(buf,linkBuf,linkBufLen);
            if (rc != linkBufLen)
                break;
            linkBufLen += 256;
            linkBuf = realloc(linkBuf,linkBufLen); }
        buf[len] = ch;
        // If an error occurred, assume this is not a link
        if (rc <= 0) {
            ptr = buf+len+1;
            continue; }
        if (!--loopCount) {
            free(buf);
            free(linkBuf);
            return(NULL); }
        // Handle absolute links
        if (*linkBuf == '/') {
            if (linkBuf[rc-1] == '/')
                rc--;
            linkBuf = realloc(linkBuf,rc + strlen(buf + len) + 1);
            strcpy(linkBuf + rc,buf + len);
            free(buf);
            buf = linkBuf;
            linkBuf = malloc(linkBufLen);
            ptr = buf+1;
            continue; }
        // Handle relative links
        if (linkBuf[rc-1] == '/')
            rc--;
        i = strlen(buf + len);
        buf = realloc(buf,(ptr - buf) + rc + i + 1);
        memmove(ptr + rc,buf + len,i + 1);
        memmove(ptr,linkBuf,rc); }
    free(linkBuf);
    return(buf);
}


/*
 * Local variables:
 * mode: c
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 */
